/**
 * Copyright 2024 SCOOP Software GmbH, cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import {
  ApiAppointmentBookingType,
  ApiAppointmentSummary,
  ApiAppointmentType,
} from "@eshg/citizen-portal-api/travelMedicine";
import { formatDate, formatTime } from "@eshg/lib-portal/formatters/dateTime";
import { durationBetweenDatesInMinutes } from "@eshg/lib-portal/helpers/dateTime";
import {
  ChevronRightOutlined,
  DateRangeOutlined,
  EventAvailableOutlined,
  EventBusyOutlined,
  EventOutlined,
  PeopleOutlined,
  VaccinesOutlined,
  WatchLaterOutlined,
} from "@mui/icons-material";
import { Button, Sheet } from "@mui/joy";
import { useRouter } from "next/navigation";

import { theme } from "@/lib/baseModule/theme/theme";
import { AppointmentOverviewButtonElement } from "@/lib/businessModules/travelMedicine/components/viewAppointment/AppointmentOverviewButtonElement";
import {
  AppointmentOverviewSection,
  AppointmentOverviewSectionGrid,
  AppointmentOverviewSectionText,
  AppointmentOverviewSectionTitle,
} from "@/lib/businessModules/travelMedicine/components/viewAppointment/AppointmentOverviewSection";
import { useCitizenRoutes } from "@/lib/businessModules/travelMedicine/shared/routes";
import { useTranslation } from "@/lib/i18n/client";
import { useAccessCodeParam } from "@/lib/shared/helpers/accessCode";
import { useIsMobile } from "@/lib/shared/hooks/useIsMobile";

interface AppointmentOverviewSheetButtonProps {
  index: number;
  procedureId: string;
  appointment: ApiAppointmentSummary;
}

export function AppointmentOverviewSheetButton(
  props: Readonly<AppointmentOverviewSheetButtonProps>,
) {
  const router = useRouter();
  const citizenRoutes = useCitizenRoutes();
  const accessCode = useAccessCodeParam();
  const isMobile = useIsMobile();
  const { t } = useTranslation(["travelMedicine/appointmentOverview"]);

  function navigateToDetails(procedureId: string, procedureStepId: string) {
    const url = `${citizenRoutes.viewAppointment.details.index(accessCode)}?procedureId=${procedureId}&procedureStepId=${procedureStepId}`;
    router.push(url);
  }

  function isCancelled() {
    return (
      props.appointment.appointmentBookingType ===
      ApiAppointmentBookingType.Cancelled
    );
  }

  function isBooked() {
    return (
      props.appointment.appointmentBookingType ===
        ApiAppointmentBookingType.AppointmentBlock ||
      props.appointment.appointmentBookingType ===
        ApiAppointmentBookingType.UserDefined
    );
  }

  return (
    <Button
      color={"neutral"}
      variant={"plain"}
      sx={{ padding: 0 }}
      onClick={() =>
        navigateToDetails(props.procedureId, props.appointment.procedureStepId)
      }
    >
      <Sheet
        component="section"
        sx={{
          display: "flex",
          flexGrow: 1,
          [theme.breakpoints.down("sm")]: {
            borderRadius: 0,
          },
          "&:hover": {
            backgroundColor: "#F0F4F8",
          },
        }}
      >
        <AppointmentOverviewSectionGrid>
          <AppointmentOverviewButtonElement
            icon={<DateRangeOutlined />}
            testId={`appointment-date-${props.index}`}
            text={formatDate(
              props.appointment.start ?? props.appointment.earliestDate,
            )}
          />
          {props.appointment.start && (
            <AppointmentOverviewSection icon={<WatchLaterOutlined />}>
              <AppointmentOverviewSectionTitle
                data-testid={`appointment-time-${props.index}`}
              >
                {t("appointmentCard.start", {
                  time: formatTime(props.appointment.start),
                })}
              </AppointmentOverviewSectionTitle>
              {props.appointment.start && props.appointment.end && (
                <AppointmentOverviewSectionText
                  data-testid={`appointment-duration-${props.index}`}
                >
                  {t("appointmentCard.duration", {
                    appointmentDuration: durationBetweenDatesInMinutes(
                      props.appointment.start,
                      props.appointment.end,
                    ),
                  })}
                </AppointmentOverviewSectionText>
              )}
            </AppointmentOverviewSection>
          )}
          {props.appointment.appointmentType ===
          ApiAppointmentType.Consultation ? (
            <AppointmentOverviewButtonElement
              icon={<PeopleOutlined />}
              testId={`appointment-type-${props.index}`}
              text={t("appointmentCard.appointmentType.consultation")}
            />
          ) : (
            <AppointmentOverviewButtonElement
              icon={<VaccinesOutlined />}
              testId={`appointment-type-${props.index}`}
              text={t("appointmentCard.appointmentType.vaccination")}
            />
          )}
          {isCancelled() ? (
            <AppointmentOverviewButtonElement
              icon={<EventBusyOutlined />}
              testId={`appointment-status-${props.index}`}
              text={t("appointmentStatus.cancelled")}
            />
          ) : isBooked() ? (
            <AppointmentOverviewButtonElement
              icon={<EventAvailableOutlined />}
              testId={`appointment-status-${props.index}`}
              text={t("appointmentStatus.booked")}
            />
          ) : (
            <AppointmentOverviewButtonElement
              icon={<EventOutlined />}
              testId={`appointment-status-${props.index}`}
              text={t("appointmentStatus.nonBooked")}
            />
          )}
        </AppointmentOverviewSectionGrid>
        <ChevronRightOutlined
          color={"primary"}
          size={"md"}
          sx={{
            alignSelf: isMobile ? "start" : "center",
          }}
        />
      </Sheet>
    </Button>
  );
}
