/**
 * Copyright 2024 cronn GmbH
 * SPDX-License-Identifier: Apache-2.0
 */

"use server";

import { ApiUserRole } from "@eshg/employee-portal-api/base";
import {
  ApiApprovalRequest,
  ApiCreateApprovalRequestRequest,
  ApiCreateManualProgressEntryRequest,
  ApiFileMetaData,
  ApiGetDetailedProcedureResponse,
  ApiGetFile200Response,
  ApiGetProcedureApprovalRequestsResponse,
  ApiGetProgressEntriesResponseProgressEntriesInner,
  ApiGetProgressEntryResponse,
  ApiGetProgressEntryResponseRelatedKeyDocumentProgressEntriesInner,
  ApiManualProgressEntry,
  ApiPatchManualProgressEntryRequest,
  ApiProgressEntry,
  ApiProgressEntryClass,
  ApiProgressEntryReferenceFilePair,
  ApiUpdateFileMetaDataRequest,
  GetProgressEntriesRequest,
} from "@eshg/employee-portal-api/businessProcedures";
import {
  UseMutationResult,
  UseSuspenseQueryResult,
} from "@tanstack/react-query";

import { UseFilterSettings } from "@/lib/shared/components/filterSettings/useFilterSettings";
import { SearchParams } from "@/lib/shared/helpers/searchParams";

export type DynamicRoute = (fileId: string) => string;

export interface EntryDeletionModalProps {
  onSuccessfulDeletion: () => void;
}

export type ProgressEntriesSearchParams = Omit<
  GetProgressEntriesRequest,
  "procedureId"
>;

interface Routes {
  entryDetails: (procedureId: string, entryId: string) => string;
  progressEntries: (procedureId: string) => string;
}

export interface ProgressEntriesPageProps extends ProgressEntryApiMethods {
  urlParams: ProgressEntriesUrlParams;
  routes: Routes;
  leaderRole: ApiUserRole;
  systemProgressEntryTypes: Record<string, string>;
  groupName: string;
  additionalKeyDocumentTypes?: Record<string, string>;
  getInitOverrides?: (inspectionId?: string) => RequestInit;
}

export interface ProgressEntriesConfig extends ProgressEntryApiActions {
  procedureId: string;
  progressEntries: ApiGetProgressEntriesResponseProgressEntriesInner[];
  detailedProcedure: ApiGetDetailedProcedureResponse;
  files: ApiProgressEntryReferenceFilePair[];
  searchParams: ProgressEntriesSearchParams;
  filterSettings: UseFilterSettings;
  routes: Routes;
  leaderRole: ApiUserRole;
  keyDocumentTypes: Record<string, string>;
  approvalRequestsResponse?: ApiGetProcedureApprovalRequestsResponse;
}

export interface ProgressEntriesUrlParams {
  params: Readonly<{ id: string; entryId?: string }>;
  searchParams: SearchParams;
}

interface ProgressEntryApiActions {
  useCreateProgressEntry: () => UseMutationResult<
    ApiManualProgressEntry,
    Error,
    {
      request: ApiCreateManualProgressEntryRequest;
      file?: File | undefined;
      fileMetaData?: ApiFileMetaData | undefined;
    },
    unknown
  >;
  useDeleteFile: () => UseMutationResult<void, Error, string, unknown>;
  useDeleteProgressEntry: () => UseMutationResult<void, Error, string, unknown>;

  usePatchProgressEntry(): UseMutationResult<
    {
      entry?: ApiManualProgressEntry;
      file?: ApiGetFile200Response;
    },
    Error,
    {
      entryId: string;
      patchProgressEntryRequest?: ApiPatchManualProgressEntryRequest;
      fileId?: string;
      updateFileMetaDataRequest?: ApiUpdateFileMetaDataRequest;
    },
    unknown
  >;

  useRequestProgressEntryDeletion: () => UseMutationResult<
    ApiApprovalRequest,
    Error,
    {
      entryId: string;
      createApprovalRequest: ApiCreateApprovalRequestRequest;
    },
    unknown
  >;
  useRequestFileDeletion: () => UseMutationResult<
    ApiApprovalRequest,
    Error,
    {
      fileId: string;
      createApprovalRequest: ApiCreateApprovalRequestRequest;
    },
    unknown
  >;
  useDecideApprovalRequest: () => UseMutationResult<
    void,
    Error,
    {
      approvalRequestId: string;
      decision: string;
    },
    unknown
  >;
  useGrantDeletionForAllRequests: () => UseMutationResult<
    void,
    Error,
    ApiApprovalRequest[],
    unknown
  >;
  useDownloadFile: () => (fileId: string) => Promise<File>;
  useGetManualProgressEntryHistory: (
    entryId: string,
  ) => UseSuspenseQueryResult<HistoryItem[] | undefined, Error>;
  useGetMetaDataHistory: (
    fileId: string,
  ) => UseSuspenseQueryResult<HistoryItem[] | undefined, Error>;
  useFetchProgressEntryDetails: (
    procedureId: string,
    progressEntryId: string,
  ) => UseSuspenseQueryResult<ApiGetProgressEntryResponse, Error>;
}

interface ProgressEntryApiMethods extends ProgressEntryApiActions {
  useFetchProgressEntries: (
    procedureId: string,
    leaderRole: ApiUserRole,
    progressEntryFilter: ProgressEntriesFilters,
  ) => UseSuspenseQueryResult<
    {
      detailedProcedure: ApiGetDetailedProcedureResponse;
      files: ApiProgressEntryReferenceFilePair[];
      progressEntries: ApiGetProgressEntriesResponseProgressEntriesInner[];
      approvalRequestsResponse:
        | ApiGetProcedureApprovalRequestsResponse
        | undefined;
    },
    Error
  >;
  useFetchProgressEntryDetails: (
    procedureId: string,
    progressEntryId: string,
  ) => UseSuspenseQueryResult<ApiGetProgressEntryResponse, Error>;
}

export interface HistoryItem {
  changedAt: Date;
  text: string | undefined;
}

export interface ProgressEntriesFilters {
  initiatedBy?: Set<string>;
  progressEntryType?: Set<string>;
  progressEntryClass?: Set<ApiProgressEntryClass>;
}

interface RelatedEntry extends Omit<ApiProgressEntry, "fileReference"> {
  fileReference: ApiGetFile200Response;
  keyDocumentVersion: number;
}

export type RelatedProgressEntry =
  ApiGetProgressEntryResponseRelatedKeyDocumentProgressEntriesInner &
    RelatedEntry;
